"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const cheerio = __importStar(require("cheerio"));
function getNodeType(tag) {
    switch (tag) {
        case "h1":
            return "heading-1";
        case "h2":
            return "heading-2";
        case "h3":
            return "heading-3";
        case "h4":
            return "heading-4";
        case "h5":
            return "heading-5";
        case "h6":
            return "heading-6";
        case "p":
        case "div":
            return "paragraph";
        case "ul":
            return "unordered-list";
        case "ol":
            return "ordered-list";
        case "li":
            return "list-item";
        case "a":
            return "hyperlink";
        case "span":
        case "b":
        case "i":
        case null:
            return "text";
        default:
            return "text";
    }
}
function isHeaderType(type) {
    switch (type) {
        case "heading-1":
        case "heading-2":
        case "heading-3":
        case "heading-4":
        case "heading-5":
        case "heading-6":
            return true;
        default:
            return false;
    }
}
function isListType(type) {
    switch (type) {
        case "ordered-list":
        case "unordered-list":
            return true;
        default:
            return false;
    }
}
function isNodeType(type) {
    switch (type) {
        case "paragraph":
        case "unordered-list":
        case "ordered-list":
        case "list-item":
            return true;
        default:
            return false;
    }
}
function generateHyperlink(root, node, parentNode, parentMarks) {
    let _nextMark = parentMarks ? [...parentMarks] : [];
    let httpNode = new ContentContentfulNode("hyperlink");
    httpNode.content.push(...generateDataForSubnodes(root, node.children, parentNode, _nextMark));
    httpNode.data = new UriData(node.attribs["href"]);
    return httpNode;
}
function getMarksFromNode(node) {
    let marks = [];
    switch (node.tagName) {
        case "b":
            marks.push(new ContentfulMark("bold"));
            break;
        case "i":
            marks.push(new ContentfulMark("italic"));
            break;
        case "u":
            marks.push(new ContentfulMark("italic"));
            break;
        case "code":
            marks.push(new ContentfulMark("code"));
            break;
    }
    return marks;
}
function generateDataForSubnodes(root, nodes, parentNode, parentMarks) {
    let leafNodes = [];
    nodes.forEach(node => {
        let _nextMark = parentMarks ? [...parentMarks] : [];
        let type = getNodeType(node.tagName);
        _nextMark.push(...getMarksFromNode(node));
        if (type == "text" && !isListType(parentNode.nodeType)) {
            if (node.tagName) {
                leafNodes.push(...generateDataForSubnodes(root, node.children, parentNode, _nextMark));
            }
            else {
                let leafNode = new LeafContentfulNode(node.data || "");
                leafNode.marks = _nextMark;
                leafNodes.push(leafNode);
            }
        }
        else if (type == "hyperlink") {
            leafNodes.push(generateHyperlink(root, node, parentNode, _nextMark));
        }
        else if (type == "list-item") {
            let listItem = new ContentContentfulNode("list-item");
            node.children.forEach(node => {
                generateDataForNode(listItem, node);
            });
            leafNodes.push(listItem);
        }
        else {
            generateDataForNode(root, node);
        }
    });
    return leafNodes;
}
function generateDataForNode(root, node) {
    let nodeType = getNodeType(node.tagName);
    let cfNode = null;
    if (isHeaderType(nodeType) || nodeType == "paragraph") {
        cfNode = new ContentContentfulNode(nodeType);
        cfNode.content.push(...generateDataForSubnodes(root, node.children, cfNode));
    }
    else if (isListType(nodeType)) {
        cfNode = new ContentContentfulNode(nodeType);
        cfNode.content.push(...generateDataForSubnodes(root, node.children, cfNode));
    }
    else if (nodeType == "text") {
        cfNode = new ContentContentfulNode("paragraph");
        if (!node.tagName) {
            cfNode.content.push(new LeafContentfulNode(node.data || ""));
        }
        else {
            cfNode.content.push(...generateDataForSubnodes(root, node.children, cfNode));
        }
    }
    else if (nodeType == "hyperlink") {
        cfNode = new ContentContentfulNode("paragraph");
        cfNode.content.push(generateHyperlink(root, node, cfNode));
    }
    if (cfNode)
        root.content.push(cfNode);
    return cfNode;
}
class BaseContentfulNode {
    constructor(nodeType = "paragraph") {
        this.nodeType = nodeType;
        this.data = new BaseData();
    }
}
exports.BaseContentfulNode = BaseContentfulNode;
class ContentContentfulNode extends BaseContentfulNode {
    constructor(nodeType = "paragraph") {
        super(nodeType);
        this.nodeType = nodeType;
        this.content = [];
    }
}
exports.ContentContentfulNode = ContentContentfulNode;
class RootContentfulNode extends ContentContentfulNode {
    constructor() {
        super("document");
    }
}
exports.RootContentfulNode = RootContentfulNode;
class LeafContentfulNode extends BaseContentfulNode {
    constructor(value, nodeType = "text") {
        super(nodeType);
        this.value = value;
        this.nodeType = nodeType;
        this.marks = [];
    }
    addMark(mark) {
        this.marks.push(new ContentfulMark(mark));
    }
}
exports.LeafContentfulNode = LeafContentfulNode;
class BaseData {
}
exports.BaseData = BaseData;
class UriData extends BaseData {
    constructor(uri = "") {
        super();
        this.uri = uri;
    }
}
exports.UriData = UriData;
class ContentfulMark {
    constructor(type = "bold") {
        this.type = type;
    }
}
exports.ContentfulMark = ContentfulMark;
function generateRichText(html) {
    let document = cheerio.parseHTML(html);
    let rich = new RootContentfulNode();
    document.forEach(doc => {
        generateDataForNode(rich, doc);
    });
    return rich;
}
exports.generateRichText = generateRichText;
//# sourceMappingURL=index.js.map